import pathlib
from typing import Any, Optional

import pytest

from competitive_verifier.oj.tools.test_command import check_gnu_time

from ..types import ConfigDirSetter, FilePaths
from .integration_data import IntegrationData


class UserDefinedAndPythonData(IntegrationData):
    def __init__(
        self,
        monkeypatch: pytest.MonkeyPatch,
        set_config_dir: ConfigDirSetter,
        file_paths: FilePaths,
    ) -> None:
        super().__init__(monkeypatch, set_config_dir, file_paths)

    @property
    def bundle_euc_ke_path(self):
        return self.config_dir_path / "bundled/encoding/EUC-KR.txt"

    @property
    def bundle_cp932_path(self):
        return self.config_dir_path / "bundled/encoding/cp932.txt"

    @property
    def config_path(self) -> Optional[str]:
        return (self.targets_path / "config.toml").as_posix()

    @property
    def exclude_path(self) -> Optional[list[str]]:
        return ["dummy/"]

    def assert_extra(self):
        assert self.bundle_euc_ke_path.read_bytes().strip() == b"cp949"
        assert self.bundle_cp932_path.read_bytes().strip() == b"cp932"

    def expected_verify_json(self) -> dict[str, Any]:
        return dict(
            {
                "files": {
                    "awk/aplusb.awk": {
                        "additonal_sources": [],
                        "dependencies": ["awk/aplusb.awk"],
                        "document_attributes": {"TITLE": 'Calculate "A + ' 'B"'},
                        "verification": [],
                    },
                    "awk/aplusb.test.awk": {
                        "additonal_sources": [],
                        "dependencies": ["awk/aplusb.test.awk"],
                        "document_attributes": {
                            "PROBLEM": "https://judge.yosupo.jp/problem/aplusb"
                        },
                        "verification": [
                            {
                                "command": {
                                    "command": f"awk -f {pathlib.Path('awk/aplusb.test.awk')}",
                                    "env": {"AWKPATH": str(self.targets_path)},
                                },
                                "compile": f"ls {self.config_dir_path/'cache/problems/8e3916c7805235eb07ec2a58660d89c6'}",
                                "name": "awk",
                                "problem": "https://judge.yosupo.jp/problem/aplusb",
                                "type": "problem",
                            }
                        ],
                    },
                    "awk/aplusb_direct.awk": {
                        "additonal_sources": [],
                        "dependencies": ["awk/aplusb_direct.awk"],
                        "document_attributes": {
                            "PROBLEM": "https://judge.yosupo.jp/problem/aplusb"
                        },
                        "verification": [
                            {
                                "command": {
                                    "command": f"awk -f {pathlib.Path('awk/aplusb_direct.awk')}",
                                    "env": {"AWKPATH": str(self.targets_path)},
                                },
                                "compile": f"ls {self.config_dir_path/'cache/problems/8e3916c7805235eb07ec2a58660d89c6'}",
                                "name": "awk",
                                "problem": "https://judge.yosupo.jp/problem/aplusb",
                                "type": "problem",
                            }
                        ],
                    },
                    "encoding/EUC-KR.txt": {
                        "dependencies": ["encoding/cp932.txt", "encoding/EUC-KR.txt"],
                        "verification": [],
                        "document_attributes": {},
                        "additonal_sources": [
                            {
                                "name": "bundled",
                                "path": self.bundle_euc_ke_path.as_posix(),
                            }
                        ],
                    },
                    "encoding/cp932.txt": {
                        "dependencies": ["encoding/cp932.txt", "encoding/EUC-KR.txt"],
                        "verification": [],
                        "document_attributes": {},
                        "additonal_sources": [
                            {
                                "name": "bundled",
                                "path": self.bundle_cp932_path.as_posix(),
                            }
                        ],
                    },
                    "python/lib_all_failure.py": {
                        "dependencies": ["python/lib_all_failure.py"],
                        "verification": [],
                        "document_attributes": {"links": []},
                        "additonal_sources": [],
                    },
                    "python/lib_some_failure.py": {
                        "dependencies": ["python/lib_some_failure.py"],
                        "verification": [],
                        "document_attributes": {"TITLE": "Units📏", "links": []},
                        "additonal_sources": [],
                    },
                    "python/lib_hidden.py": {
                        "additonal_sources": [],
                        "dependencies": ["python/lib_hidden.py"],
                        "document_attributes": {"DISPLAY": "hidden", "links": []},
                        "verification": [],
                    },
                    "python/lib_skip.py": {
                        "dependencies": ["python/lib_skip.py"],
                        "verification": [],
                        "document_attributes": {"links": []},
                        "additonal_sources": [],
                    },
                    "python/lib_some_skip_some_wa.py": {
                        "dependencies": ["python/lib_some_skip_some_wa.py"],
                        "verification": [],
                        "document_attributes": {"links": []},
                        "additonal_sources": [],
                    },
                    "python/success1.py": {
                        "dependencies": [
                            "python/lib_some_failure.py",
                            "python/lib_some_skip.py",
                            "python/success1.py",
                        ],
                        "verification": [
                            {
                                "name": "Python",
                                "type": "problem",
                                "command": f"env PYTHONPATH={self.targets_path.as_posix()} python python/success1.py",
                                "problem": "https://judge.yosupo.jp/problem/aplusb",
                            }
                        ],
                        "document_attributes": {
                            "PROBLEM": "https://judge.yosupo.jp/problem/aplusb",
                            "links": ["https://judge.yosupo.jp/problem/aplusb"],
                        },
                        "additonal_sources": [],
                    },
                    "python/failure.wa.py": {
                        "dependencies": [
                            "python/failure.wa.py",
                            "python/lib_all_failure.py",
                            "python/lib_some_skip_some_wa.py",
                        ],
                        "verification": [
                            {
                                "name": "Python",
                                "type": "problem",
                                "command": f"env PYTHONPATH={self.targets_path.as_posix()} python python/failure.wa.py",
                                "problem": "https://judge.yosupo.jp/problem/aplusb",
                            }
                        ],
                        "document_attributes": {
                            "PROBLEM": "https://judge.yosupo.jp/problem/aplusb",
                            "links": ["https://judge.yosupo.jp/problem/aplusb"],
                        },
                        "additonal_sources": [],
                    },
                    "python/failure.mle.py": {
                        "dependencies": [
                            "python/failure.mle.py",
                            "python/lib_all_failure.py",
                            "python/lib_some_failure.py",
                        ],
                        "verification": [
                            {
                                "name": "Python",
                                "type": "problem",
                                "command": f"env PYTHONPATH={self.targets_path.as_posix()} python python/failure.mle.py",
                                "problem": "https://judge.yosupo.jp/problem/aplusb",
                                "mle": 100.0,
                            }
                        ],
                        "document_attributes": {
                            "PROBLEM": "https://judge.yosupo.jp/problem/aplusb",
                            "MLE": "100",
                            "links": ["https://judge.yosupo.jp/problem/aplusb"],
                        },
                        "additonal_sources": [],
                    },
                    "python/skip.py": {
                        "dependencies": [
                            "python/lib_skip.py",
                            "python/lib_some_skip_some_wa.py",
                            "python/skip.py",
                        ],
                        "verification": [{"type": "const", "status": "skipped"}],
                        "document_attributes": {
                            "PROBLEM": "https://judge.yosupo.jp/problem/aplusb",
                            "IGNORE": "",
                            "links": ["https://judge.yosupo.jp/problem/aplusb"],
                        },
                        "additonal_sources": [],
                    },
                    "python/success2.py": {
                        "dependencies": [
                            "python/lib_all_success.py",
                            "python/lib_some_skip_some_wa.py",
                            "python/success2.py",
                        ],
                        "verification": [
                            {
                                "name": "Python",
                                "type": "problem",
                                "command": f"env PYTHONPATH={self.targets_path.as_posix()} python python/success2.py",
                                "problem": "https://judge.yosupo.jp/problem/aplusb",
                            }
                        ],
                        "document_attributes": {
                            "PROBLEM": "https://judge.yosupo.jp/problem/aplusb",
                            "links": ["https://judge.yosupo.jp/problem/aplusb"],
                        },
                        "additonal_sources": [],
                    },
                    "python/lib_all_success.py": {
                        "dependencies": ["python/lib_all_success.py"],
                        "verification": [],
                        "document_attributes": {"links": []},
                        "additonal_sources": [],
                    },
                    "python/failure.re.py": {
                        "dependencies": [
                            "python/failure.re.py",
                            "python/lib_all_failure.py",
                        ],
                        "verification": [
                            {
                                "name": "Python",
                                "type": "problem",
                                "command": f"env PYTHONPATH={self.targets_path.as_posix()} python python/failure.re.py",
                                "problem": "https://judge.yosupo.jp/problem/aplusb",
                            }
                        ],
                        "document_attributes": {
                            "PROBLEM": "https://judge.yosupo.jp/problem/aplusb",
                            "links": ["https://judge.yosupo.jp/problem/aplusb"],
                        },
                        "additonal_sources": [],
                    },
                    "python/lib_some_skip.py": {
                        "dependencies": ["python/lib_some_skip.py"],
                        "verification": [],
                        "document_attributes": {"links": []},
                        "additonal_sources": [],
                    },
                    "python/failure.tle.py": {
                        "dependencies": [
                            "python/failure.tle.py",
                            "python/lib_all_failure.py",
                        ],
                        "verification": [
                            {
                                "name": "Python",
                                "type": "problem",
                                "command": f"env PYTHONPATH={self.targets_path.as_posix()} python python/failure.tle.py",
                                "problem": "https://onlinejudge.u-aizu.ac.jp/courses/lesson/2/ITP1/1/ITP1_1_A",
                                "tle": 0.1,
                            }
                        ],
                        "document_attributes": {
                            "PROBLEM": "https://onlinejudge.u-aizu.ac.jp/courses/lesson/2/ITP1/1/ITP1_1_A",
                            "TLE": "0.1",
                            "links": [
                                "https://onlinejudge.u-aizu.ac.jp/courses/lesson/2/ITP1/1/ITP1_1_A"
                            ],
                        },
                        "additonal_sources": [],
                    },
                    "python/sub/no_dependants.py": {
                        "additonal_sources": [],
                        "dependencies": ["python/sub/no_dependants.py"],
                        "document_attributes": {"links": []},
                        "verification": [],
                    },
                }
            }
        )

    def expected_verify_result(self) -> dict[str, Any]:
        return {
            "total_seconds": 11189.039999999999,
            "files": {
                "awk/aplusb.test.awk": {
                    "newest": True,
                    "verifications": [
                        {
                            "elapsed": 6605.0,
                            "heaviest": 706.0,
                            "last_execution_time": "2021-04-10T08:08:54.390000+02:00",
                            "slowest": 660.0,
                            "status": "failure",
                            "testcases": [
                                {
                                    "elapsed": 3.96,
                                    "memory": 10.63,
                                    "name": "example_00",
                                    "status": "RE",
                                },
                                {
                                    "elapsed": 4.87,
                                    "memory": 57.66,
                                    "name": "example_01",
                                    "status": "RE",
                                },
                                {
                                    "elapsed": 9.41,
                                    "memory": 55.5,
                                    "name": "random_00",
                                    "status": "RE",
                                },
                                {
                                    "elapsed": 8.94,
                                    "memory": 5.09,
                                    "name": "random_01",
                                    "status": "RE",
                                },
                                {
                                    "elapsed": 6.02,
                                    "memory": 50.45,
                                    "name": "random_02",
                                    "status": "RE",
                                },
                                {
                                    "elapsed": 0.29,
                                    "memory": 48.46,
                                    "name": "random_03",
                                    "status": "RE",
                                },
                                {
                                    "elapsed": 7.92,
                                    "memory": 57.15,
                                    "name": "random_04",
                                    "status": "RE",
                                },
                                {
                                    "elapsed": 2.76,
                                    "memory": 6.76,
                                    "name": "random_05",
                                    "status": "RE",
                                },
                                {
                                    "elapsed": 8.25,
                                    "memory": 23.66,
                                    "name": "random_06",
                                    "status": "RE",
                                },
                                {
                                    "elapsed": 6.99,
                                    "memory": 66.49,
                                    "name": "random_07",
                                    "status": "RE",
                                },
                                {
                                    "elapsed": 8.71,
                                    "memory": 16.96,
                                    "name": "random_08",
                                    "status": "RE",
                                },
                                {
                                    "elapsed": 5.17,
                                    "memory": 36.25,
                                    "name": "random_09",
                                    "status": "RE",
                                },
                            ],
                            "verification_name": "awk",
                        }
                    ],
                },
                "awk/aplusb_direct.awk": {
                    "newest": True,
                    "verifications": [
                        {
                            "elapsed": 2481.0,
                            "heaviest": 198.0,
                            "last_execution_time": "2019-09-03T13:45:49.350000-02:00",
                            "slowest": 248.0,
                            "status": "success",
                            "testcases": [
                                {
                                    "elapsed": 3.82,
                                    "memory": 48.12,
                                    "name": "example_00",
                                    "status": "AC",
                                },
                                {
                                    "elapsed": 6.27,
                                    "memory": 69.16,
                                    "name": "example_01",
                                    "status": "AC",
                                },
                                {
                                    "elapsed": 8.99,
                                    "memory": 97.03,
                                    "name": "random_00",
                                    "status": "AC",
                                },
                                {
                                    "elapsed": 7.9,
                                    "memory": 25.28,
                                    "name": "random_01",
                                    "status": "AC",
                                },
                                {
                                    "elapsed": 5.37,
                                    "memory": 77.2,
                                    "name": "random_02",
                                    "status": "AC",
                                },
                                {
                                    "elapsed": 4.71,
                                    "memory": 95.31,
                                    "name": "random_03",
                                    "status": "AC",
                                },
                                {
                                    "elapsed": 0.02,
                                    "memory": 3.83,
                                    "name": "random_04",
                                    "status": "AC",
                                },
                                {
                                    "elapsed": 8.69,
                                    "memory": 55.61,
                                    "name": "random_05",
                                    "status": "AC",
                                },
                                {
                                    "elapsed": 4.75,
                                    "memory": 46.8,
                                    "name": "random_06",
                                    "status": "AC",
                                },
                                {
                                    "elapsed": 8.08,
                                    "memory": 75.38,
                                    "name": "random_07",
                                    "status": "AC",
                                },
                                {
                                    "elapsed": 9.96,
                                    "memory": 58.54,
                                    "name": "random_08",
                                    "status": "AC",
                                },
                                {
                                    "elapsed": 7.36,
                                    "memory": 98.49,
                                    "name": "random_09",
                                    "status": "AC",
                                },
                            ],
                            "verification_name": "awk",
                        }
                    ],
                },
                "python/failure.mle.py": {
                    "verifications": [
                        {
                            "verification_name": "Python",
                            "status": "failure",
                            "elapsed": 4426.0,
                            "last_execution_time": "1973-10-02T03:07:14.120000Z",
                            **(
                                {
                                    "slowest": 442.0,
                                    "heaviest": 349.0,
                                    "testcases": [
                                        {
                                            "name": "example_00",
                                            "status": "MLE",
                                            "elapsed": 5.34,
                                            "memory": 89.12,
                                        },
                                        {
                                            "name": "example_01",
                                            "status": "MLE",
                                            "elapsed": 9.79,
                                            "memory": 78.31,
                                        },
                                        {
                                            "name": "random_00",
                                            "status": "MLE",
                                            "elapsed": 4.8,
                                            "memory": 6.08,
                                        },
                                        {
                                            "name": "random_01",
                                            "status": "AC",
                                            "elapsed": 4.09,
                                            "memory": 15.08,
                                        },
                                        {
                                            "name": "random_02",
                                            "status": "AC",
                                            "elapsed": 3.33,
                                            "memory": 6.99,
                                        },
                                        {
                                            "name": "random_03",
                                            "status": "MLE",
                                            "elapsed": 4.04,
                                            "memory": 9.04,
                                        },
                                        {
                                            "name": "random_04",
                                            "status": "AC",
                                            "elapsed": 8.19,
                                            "memory": 81.73,
                                        },
                                        {
                                            "name": "random_05",
                                            "status": "AC",
                                            "elapsed": 0.18,
                                            "memory": 47.13,
                                        },
                                        {
                                            "name": "random_06",
                                            "status": "AC",
                                            "elapsed": 1.01,
                                            "memory": 68.03,
                                        },
                                        {
                                            "name": "random_07",
                                            "status": "MLE",
                                            "elapsed": 3.76,
                                            "memory": 40.93,
                                        },
                                        {
                                            "name": "random_08",
                                            "status": "AC",
                                            "elapsed": 1.32,
                                            "memory": 69.99,
                                        },
                                        {
                                            "name": "random_09",
                                            "status": "MLE",
                                            "elapsed": 0.12,
                                            "memory": 19.27,
                                        },
                                    ],
                                }
                                if check_gnu_time()
                                else {}
                            ),
                        }
                    ],
                    "newest": True,
                },
                "python/failure.wa.py": {
                    "verifications": [
                        {
                            "verification_name": "Python",
                            "status": "failure",
                            "elapsed": 3240.0,
                            "slowest": 324.0,
                            "heaviest": 994.0,
                            "testcases": [
                                {
                                    "name": "example_00",
                                    "status": "AC",
                                    "elapsed": 1.89,
                                    "memory": 34.41,
                                },
                                {
                                    "name": "example_01",
                                    "status": "AC",
                                    "elapsed": 2.17,
                                    "memory": 10.24,
                                },
                                {
                                    "name": "random_00",
                                    "status": "AC",
                                    "elapsed": 3.25,
                                    "memory": 7.82,
                                },
                                {
                                    "name": "random_01",
                                    "status": "WA",
                                    "elapsed": 7.41,
                                    "memory": 81.13,
                                },
                                {
                                    "name": "random_02",
                                    "status": "WA",
                                    "elapsed": 1.53,
                                    "memory": 91.42,
                                },
                                {
                                    "name": "random_03",
                                    "status": "AC",
                                    "elapsed": 4.6,
                                    "memory": 13.13,
                                },
                                {
                                    "name": "random_04",
                                    "status": "WA",
                                    "elapsed": 5.09,
                                    "memory": 38.65,
                                },
                                {
                                    "name": "random_05",
                                    "status": "WA",
                                    "elapsed": 6.34,
                                    "memory": 73.13,
                                },
                                {
                                    "name": "random_06",
                                    "status": "AC",
                                    "elapsed": 3.13,
                                    "memory": 32.18,
                                },
                                {
                                    "name": "random_07",
                                    "status": "AC",
                                    "elapsed": 8.88,
                                    "memory": 10.07,
                                },
                                {
                                    "name": "random_08",
                                    "status": "WA",
                                    "elapsed": 1.66,
                                    "memory": 88.91,
                                },
                                {
                                    "name": "random_09",
                                    "status": "WA",
                                    "elapsed": 9.41,
                                    "memory": 34.78,
                                },
                            ],
                            "last_execution_time": "1986-10-21T03:14:48.980000+11:00",
                        }
                    ],
                    "newest": True,
                },
                "python/failure.tle.py": {
                    "verifications": [
                        {
                            "verification_name": "Python",
                            "status": "failure",
                            "elapsed": 1020.0,
                            "slowest": 102.0,
                            "heaviest": 966.0,
                            "testcases": [
                                {
                                    "name": "judge_data",
                                    "status": "TLE",
                                    "elapsed": 6.75,
                                    "memory": 8.22,
                                }
                            ],
                            "last_execution_time": "2054-12-13T14:49:21.790000-08:00",
                        }
                    ],
                    "newest": True,
                },
                "python/success1.py": {
                    "verifications": [
                        {
                            "verification_name": "Python",
                            "status": "success",
                            "elapsed": 8685.0,
                            "slowest": 868.0,
                            "heaviest": 955.0,
                            "testcases": [
                                {
                                    "name": "example_00",
                                    "status": "AC",
                                    "elapsed": 8.66,
                                    "memory": 14.97,
                                },
                                {
                                    "name": "example_01",
                                    "status": "AC",
                                    "elapsed": 3.4,
                                    "memory": 82.48,
                                },
                                {
                                    "name": "random_00",
                                    "status": "AC",
                                    "elapsed": 6.01,
                                    "memory": 69.23,
                                },
                                {
                                    "name": "random_01",
                                    "status": "AC",
                                    "elapsed": 8.47,
                                    "memory": 16.15,
                                },
                                {
                                    "name": "random_02",
                                    "status": "AC",
                                    "elapsed": 1.28,
                                    "memory": 88.6,
                                },
                                {
                                    "name": "random_03",
                                    "status": "AC",
                                    "elapsed": 5.69,
                                    "memory": 85.15,
                                },
                                {
                                    "name": "random_04",
                                    "status": "AC",
                                    "elapsed": 2.52,
                                    "memory": 74.99,
                                },
                                {
                                    "name": "random_05",
                                    "status": "AC",
                                    "elapsed": 2.0,
                                    "memory": 4.66,
                                },
                                {
                                    "name": "random_06",
                                    "status": "AC",
                                    "elapsed": 4.46,
                                    "memory": 13.48,
                                },
                                {
                                    "name": "random_07",
                                    "status": "AC",
                                    "elapsed": 7.96,
                                    "memory": 6.91,
                                },
                                {
                                    "name": "random_08",
                                    "status": "AC",
                                    "elapsed": 7.67,
                                    "memory": 39.47,
                                },
                                {
                                    "name": "random_09",
                                    "status": "AC",
                                    "elapsed": 0.16,
                                    "memory": 36.45,
                                },
                            ],
                            "last_execution_time": "2034-09-03T09:15:32.640000+02:00",
                        }
                    ],
                    "newest": True,
                },
                "python/success2.py": {
                    "verifications": [
                        {
                            "verification_name": "Python",
                            "status": "success",
                            "elapsed": 735.0,
                            "slowest": 73.0,
                            "heaviest": 19.0,
                            "testcases": [
                                {
                                    "name": "example_00",
                                    "status": "AC",
                                    "elapsed": 1.1,
                                    "memory": 53.93,
                                },
                                {
                                    "name": "example_01",
                                    "status": "AC",
                                    "elapsed": 5.48,
                                    "memory": 50.39,
                                },
                                {
                                    "name": "random_00",
                                    "status": "AC",
                                    "elapsed": 9.31,
                                    "memory": 58.92,
                                },
                                {
                                    "name": "random_01",
                                    "status": "AC",
                                    "elapsed": 4.3,
                                    "memory": 35.83,
                                },
                                {
                                    "name": "random_02",
                                    "status": "AC",
                                    "elapsed": 2.64,
                                    "memory": 87.34,
                                },
                                {
                                    "name": "random_03",
                                    "status": "AC",
                                    "elapsed": 2.73,
                                    "memory": 54.56,
                                },
                                {
                                    "name": "random_04",
                                    "status": "AC",
                                    "elapsed": 5.98,
                                    "memory": 34.29,
                                },
                                {
                                    "name": "random_05",
                                    "status": "AC",
                                    "elapsed": 3.52,
                                    "memory": 91.97,
                                },
                                {
                                    "name": "random_06",
                                    "status": "AC",
                                    "elapsed": 7.15,
                                    "memory": 79.87,
                                },
                                {
                                    "name": "random_07",
                                    "status": "AC",
                                    "elapsed": 6.63,
                                    "memory": 11.11,
                                },
                                {
                                    "name": "random_08",
                                    "status": "AC",
                                    "elapsed": 6.53,
                                    "memory": 74.19,
                                },
                                {
                                    "name": "random_09",
                                    "status": "AC",
                                    "elapsed": 7.4,
                                    "memory": 27.16,
                                },
                            ],
                            "last_execution_time": "1991-06-27T16:59:27.680000+06:00",
                        }
                    ],
                    "newest": True,
                },
                "python/failure.re.py": {
                    "verifications": [
                        {
                            "verification_name": "Python",
                            "status": "failure",
                            "elapsed": 554.0,
                            "slowest": 55.0,
                            "heaviest": 10.0,
                            "testcases": [
                                {
                                    "name": "example_00",
                                    "status": "RE",
                                    "elapsed": 3.47,
                                    "memory": 55.14,
                                },
                                {
                                    "name": "example_01",
                                    "status": "RE",
                                    "elapsed": 4.8,
                                    "memory": 82.74,
                                },
                                {
                                    "name": "random_00",
                                    "status": "RE",
                                    "elapsed": 4.62,
                                    "memory": 53.04,
                                },
                                {
                                    "name": "random_01",
                                    "status": "RE",
                                    "elapsed": 5.6,
                                    "memory": 84.32,
                                },
                                {
                                    "name": "random_02",
                                    "status": "RE",
                                    "elapsed": 9.28,
                                    "memory": 18.09,
                                },
                                {
                                    "name": "random_03",
                                    "status": "RE",
                                    "elapsed": 6.41,
                                    "memory": 13.18,
                                },
                                {
                                    "name": "random_04",
                                    "status": "RE",
                                    "elapsed": 7.17,
                                    "memory": 63.97,
                                },
                                {
                                    "name": "random_05",
                                    "status": "RE",
                                    "elapsed": 7.59,
                                    "memory": 75.35,
                                },
                                {
                                    "name": "random_06",
                                    "status": "RE",
                                    "elapsed": 4.79,
                                    "memory": 50.9,
                                },
                                {
                                    "name": "random_07",
                                    "status": "RE",
                                    "elapsed": 2.83,
                                    "memory": 25.32,
                                },
                                {
                                    "name": "random_08",
                                    "status": "RE",
                                    "elapsed": 1.71,
                                    "memory": 53.67,
                                },
                                {
                                    "name": "random_09",
                                    "status": "RE",
                                    "elapsed": 4.52,
                                    "memory": 85.17,
                                },
                            ],
                            "last_execution_time": "2047-08-08T05:27:05.540000-08:00",
                        }
                    ],
                    "newest": True,
                },
                "python/skip.py": {
                    "verifications": [
                        {
                            "status": "skipped",
                            "elapsed": 5703.0,
                            "last_execution_time": "1999-08-27T06:18:53.270000-10:00",
                        }
                    ],
                    "newest": True,
                },
            },
        }



# Python Version: 3.x
import functools
import pathlib
from logging import getLogger
from typing import Optional

from pydantic import BaseModel, Field

import competitive_verifier.oj.verify.shlex2 as shlex
from competitive_verifier.oj.verify.models import (
    Language,
    LanguageEnvironment,
    OjVerifyLanguageConfig,
)
from competitive_verifier.oj.verify.utils import read_text_normalized

logger = getLogger(__name__)


class OjVerifyNimConfigEnv(BaseModel):
    compile_to: Optional[str]
    NIMFLAGS: Optional[list[str]]


class OjVerifyNimConfig(OjVerifyLanguageConfig):
    environments: list[OjVerifyNimConfigEnv] = Field(default_factory=list)


class NimLanguageEnvironment(LanguageEnvironment):
    compile_to: str
    nim_flags: list[str]

    def __init__(self, *, compile_to: str, NIMFLAGS: list[str]):
        self.compile_to = compile_to
        self.nim_flags = NIMFLAGS

    @property
    def name(self) -> str:
        return "Nim"

    def get_compile_command(
        self, path: pathlib.Path, *, basedir: pathlib.Path, tempdir: pathlib.Path
    ) -> str:
        return shlex.join(
            [
                "nim",
                self.compile_to,
                "-p:.",
                f"-o:{str(tempdir /'a.out')}",
                f"--nimcache:{str(tempdir)}",
            ]
            + self.nim_flags
            + [str(path)]
        )

    def get_execute_command(
        self, path: pathlib.Path, *, basedir: pathlib.Path, tempdir: pathlib.Path
    ) -> str:
        return str(tempdir / "a.out")


@functools.lru_cache(maxsize=None)
def _list_direct_dependencies(
    path: pathlib.Path, *, basedir: pathlib.Path
) -> list[pathlib.Path]:
    items: list[str] = []
    for line in read_text_normalized(basedir / path).splitlines():
        line = line.strip()
        if line.startswith("include"):
            items += line[7:].strip().split(",")
        elif line.startswith("import"):
            line = line[6:]
            i = line.find(" except ")
            if i >= 0:
                line = line[:i]
            items += line.split(",")
        elif line.startswith("from"):
            i = line.find(" import ")
            if i >= 0:
                items += line[4 : i - 1]
    dependencies = [path.resolve()]
    for item in items:
        item = item.strip()
        if item.startswith('"'):
            item = item[1 : len(item) - 1]
        else:
            item += ".nim"
        item_ = pathlib.Path(item)
        if item_.exists():
            dependencies.append(item_)
    return list(set(dependencies))


class NimLanguage(Language):
    config: OjVerifyNimConfig

    def __init__(self, *, config: Optional[OjVerifyNimConfig]):
        self.config = config or OjVerifyNimConfig()

    def list_dependencies(
        self, path: pathlib.Path, *, basedir: pathlib.Path
    ) -> list[pathlib.Path]:
        dependencies: list[pathlib.Path] = []
        visited: set[pathlib.Path] = set()
        stk = [path.resolve()]
        while stk:
            path = stk.pop()
            if path in visited:
                continue
            visited.add(path)
            for child in _list_direct_dependencies(path, basedir=basedir):
                dependencies.append(child)
                stk.append(child)
        return list(set(dependencies))

    def list_environments(
        self, path: pathlib.Path, *, basedir: pathlib.Path
    ) -> list[NimLanguageEnvironment]:
        default_compile_to = "cpp"
        default_NIMFLAGS = ["-d:release", "--opt:speed"]
        if self.config.environments:
            return [
                NimLanguageEnvironment(
                    compile_to=default_compile_to,
                    NIMFLAGS=default_NIMFLAGS,
                )
            ]
        else:
            return [
                NimLanguageEnvironment(
                    compile_to=env.compile_to or default_compile_to,
                    NIMFLAGS=env.NIMFLAGS or default_NIMFLAGS,
                )
                for env in self.config.environments
            ]



import argparse
import logging
import math
import pathlib
import sys
from logging import getLogger
from typing import Optional

from competitive_verifier import github, summary
from competitive_verifier.arg import (
    add_ignore_error_argument,
    add_verbose_argument,
    add_verify_files_json_argument,
    add_write_summary_argument,
)
from competitive_verifier.error import VerifierError
from competitive_verifier.log import configure_stderr_logging
from competitive_verifier.models import VerificationInput, VerifyCommandResult
from competitive_verifier.verify.verifier import SplitState, Verifier

logger = getLogger(__name__)


def run_impl(
    input: VerificationInput,
    *,
    prev_result: Optional[VerifyCommandResult],
    timeout: float = math.inf,
    default_tle: Optional[float] = None,
    default_mle: Optional[float] = None,
    download: bool = True,
    split: Optional[int] = None,
    split_index: Optional[int] = None,
    output_path: Optional[pathlib.Path] = None,
    write_summary: bool = False,
    ignore_error: bool = False,
) -> bool:
    split_state = get_split_state(split, split_index)

    if timeout == 0:
        timeout = math.inf

    verifier = Verifier(
        input,
        use_git_timestamp=github.env.is_in_github_actions(),
        timeout=timeout,
        default_tle=default_tle,
        default_mle=default_mle,
        prev_result=prev_result,
        split_state=split_state,
    )
    result = verifier.verify(download=download)
    result_json = result.model_dump_json(exclude_none=True)

    if write_summary:
        gh_summary_path = github.env.get_step_summary_path()
        if gh_summary_path and gh_summary_path.parent.exists():
            with open(gh_summary_path, "w", encoding="utf-8") as fp:
                summary.write_summary(fp, result)
        else:
            logger.warning("write_summary=True but not found $GITHUB_STEP_SUMMARY")

    print(result_json)

    if output_path:
        output_path.parent.mkdir(parents=True, exist_ok=True)
        output_path.write_text(result_json, encoding="utf-8")

    is_success = result.is_success()

    if is_success:
        logger.info("success!")
    else:
        logger.warning("not success!")

    return is_success or ignore_error


def run(args: argparse.Namespace) -> bool:
    default_level = logging.INFO
    if args.verbose:
        default_level = logging.DEBUG
    configure_stderr_logging(default_level)

    logger.debug("arguments=%s", vars(args))
    logger.info("verify_files_json=%s", str(args.verify_files_json))
    input = VerificationInput.parse_file_relative(args.verify_files_json)
    prev_result = None
    if args.prev_result:
        try:
            prev_result = VerifyCommandResult.parse_file_relative(args.prev_result)
        except Exception:
            logger.warning("Failed to parse prev_result: %s", args.prev_result)

    return run_impl(
        input,
        timeout=args.timeout,
        default_tle=args.default_tle,
        default_mle=args.default_mle,
        prev_result=prev_result,
        download=args.download,
        split=args.split,
        split_index=args.split_index,
        output_path=args.output,
        write_summary=args.write_summary,
        ignore_error=args.ignore_error,
    )


def argument(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
    add_verbose_argument(parser)
    add_verify_files_json_argument(parser)
    add_ignore_error_argument(parser)
    add_write_summary_argument(parser)
    parser.add_argument(
        "--timeout",
        type=float,
        default=math.inf,
        help="Timeout seconds. if value is zero, it is same to math.inf.",
    )
    parser.add_argument(
        "--tle",
        dest="default_tle",
        type=float,
        default=None,
        help="Threshold seconds to be TLE",
    )
    parser.add_argument(
        "--mle",
        dest="default_mle",
        type=float,
        default=None,
        help="Threshold memory usage (MB) to be MLE",
    )
    parser.add_argument(
        "--prev-result",
        type=pathlib.Path,
        required=False,
        help="Previous result json file",
    )

    parser.add_argument(
        "--no-download",
        action="store_false",
        dest="download",
        help="Suppress `oj download`",
    )
    parser.add_argument(
        "--output",
        "-o",
        type=pathlib.Path,
        required=False,
        help="The output file for which verifier saves the result json.",
    )
    parser.add_argument_group()
    parallel_group = parser.add_argument_group("parallel")
    parallel_group.add_argument(
        "--split",
        type=int,
        help="Parallel job size",
        required=False,
    )
    parallel_group.add_argument(
        "--split-index",
        type=int,
        help="Parallel job index",
        required=False,
    )

    return parser


def get_split_state(
    split: Optional[int] = None,
    split_index: Optional[int] = None,
) -> Optional[SplitState]:
    if split_index is None and split is None:
        return None

    if split_index is not None and split is not None:
        if split <= 0:
            raise VerifierError("--split must be greater than 0.")
        if not (0 <= split_index < split):
            raise VerifierError(
                "--split-index must be greater than 0 and less than --split."
            )
        return SplitState(size=split, index=split_index)

    if split is not None:
        raise VerifierError("--split argument requires --split-index argument.")

    if split_index is not None:
        raise VerifierError("--split-index argument requires --split argument.")

    raise VerifierError("invalid state.")


def main(args: Optional[list[str]] = None) -> None:
    try:
        parsed = argument(argparse.ArgumentParser()).parse_args(args)
        if not run(parsed):
            sys.exit(1)
    except Exception as e:
        sys.stderr.write(str(e))
        sys.exit(2)


if __name__ == "__main__":
    main()



import datetime
from pathlib import Path
from typing import Any
from unittest import mock

import pytest
from pydantic import RootModel

from competitive_verifier.documents.render import SourceCodeStat, StatusIcon
from competitive_verifier.models import VerificationInput, VerifyCommandResult

Parser = RootModel[dict[Path, SourceCodeStat]]

test_resolve_dependency_params: list[tuple[str, Any, Any, Any, Any]] = [
    (
        "AC",
        {
            "files": {
                "c1/a.py": {},
                "c1/b1.py": {"dependencies": ["c1/a.py", "c1/b2.py"]},
                "c1/b2.py": {"dependencies": ["c1/b1.py"]},
                "c1/test.py": {
                    "dependencies": ["c1/b1.py"],
                    "verification": [
                        {
                            "type": "const",
                            "status": "success",
                        }
                    ],
                },
            }
        },
        {
            "total_seconds": 10,
            "files": {
                "c1/test.py": {
                    "verifications": [
                        {
                            "status": "success",
                            "elapsed": 1,
                            "last_execution_time": datetime.datetime(2020, 2, 15),
                        },
                    ]
                },
            },
        },
        ["c1/a.py", "c1/b1.py", "c1/b2.py", "c1/test.py"],
        {
            "c1/test.py": {
                "path": "c1/test.py",
                "file_input": {
                    "dependencies": ["c1/b1.py"],
                    "verification": [
                        {
                            "type": "const",
                            "status": "success",
                        }
                    ],
                },
                "is_verification": True,
                "verification_status": StatusIcon.TEST_ACCEPTED,
                "verification_results": [
                    {
                        "status": "success",
                        "elapsed": 1,
                        "last_execution_time": datetime.datetime(2020, 2, 15),
                    },
                ],
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": {"c1/b1.py"},
                "required_by": set(),
                "verified_with": set(),
            },
            "c1/b1.py": {
                "path": "c1/b1.py",
                "file_input": {"dependencies": ["c1/a.py", "c1/b2.py"]},
                "is_verification": False,
                "verification_status": StatusIcon.LIBRARY_ALL_AC,
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": {"c1/b2.py", "c1/a.py"},
                "required_by": {"c1/b2.py"},
                "verified_with": {"c1/test.py"},
            },
            "c1/b2.py": {
                "path": "c1/b2.py",
                "file_input": {"dependencies": ["c1/b1.py"]},
                "is_verification": False,
                "verification_status": StatusIcon.LIBRARY_ALL_AC,
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": {"c1/b1.py"},
                "required_by": {"c1/b1.py"},
                "verified_with": set(),
            },
            "c1/a.py": {
                "path": "c1/a.py",
                "file_input": {},
                "is_verification": False,
                "verification_status": StatusIcon.LIBRARY_ALL_AC,
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": set(),
                "required_by": {"c1/b1.py"},
                "verified_with": set(),
            },
        },
    ),
    (
        "WA",
        {
            "files": {
                "c1/a.py": {},
                "c1/b1.py": {"dependencies": ["c1/a.py", "c1/b2.py"]},
                "c1/b2.py": {"dependencies": ["c1/b1.py"]},
                "c1/test.py": {
                    "dependencies": ["c1/b1.py"],
                    "verification": [
                        {
                            "type": "const",
                            "status": "failure",
                        }
                    ],
                },
            }
        },
        {
            "total_seconds": 10,
            "files": {
                "c1/test.py": {
                    "verifications": [
                        {
                            "status": "failure",
                            "elapsed": 1,
                            "last_execution_time": datetime.datetime(2020, 2, 15),
                        },
                    ]
                },
            },
        },
        ["c1/a.py", "c1/b1.py", "c1/b2.py", "c1/test.py"],
        {
            "c1/test.py": {
                "path": "c1/test.py",
                "file_input": {
                    "dependencies": ["c1/b1.py"],
                    "verification": [
                        {
                            "type": "const",
                            "status": "failure",
                        }
                    ],
                },
                "is_verification": True,
                "verification_status": StatusIcon.TEST_WRONG_ANSWER,
                "verification_results": [
                    {
                        "status": "failure",
                        "elapsed": 1,
                        "last_execution_time": datetime.datetime(2020, 2, 15),
                    },
                ],
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": {"c1/b1.py"},
                "required_by": set(),
                "verified_with": set(),
            },
            "c1/b1.py": {
                "path": "c1/b1.py",
                "file_input": {"dependencies": ["c1/a.py", "c1/b2.py"]},
                "is_verification": False,
                "verification_status": StatusIcon.LIBRARY_ALL_WA,
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": {"c1/b2.py", "c1/a.py"},
                "required_by": {"c1/b2.py"},
                "verified_with": {"c1/test.py"},
            },
            "c1/b2.py": {
                "path": "c1/b2.py",
                "file_input": {"dependencies": ["c1/b1.py"]},
                "is_verification": False,
                "verification_status": StatusIcon.LIBRARY_ALL_WA,
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": {"c1/b1.py"},
                "required_by": {"c1/b1.py"},
                "verified_with": set(),
            },
            "c1/a.py": {
                "path": "c1/a.py",
                "file_input": {},
                "is_verification": False,
                "verification_status": StatusIcon.LIBRARY_ALL_WA,
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": set(),
                "required_by": {"c1/b1.py"},
                "verified_with": set(),
            },
        },
    ),
    (
        "AC & skip",
        {
            "files": {
                "c1/a.py": {},
                "c1/b1.py": {"dependencies": ["c1/a.py", "c1/b2.py"]},
                "c1/b2.py": {"dependencies": ["c1/b1.py"]},
                "c1/test.py": {
                    "dependencies": ["c1/b1.py"],
                    "verification": [
                        {
                            "type": "const",
                            "status": "success",
                        }
                    ],
                },
                "c1/test2.py": {
                    "dependencies": ["c1/b1.py"],
                    "verification": [
                        {
                            "type": "const",
                            "status": "skipped",
                        }
                    ],
                },
            }
        },
        {
            "total_seconds": 10,
            "files": {
                "c1/test.py": {
                    "verifications": [
                        {
                            "status": "success",
                            "elapsed": 1,
                            "last_execution_time": datetime.datetime(2020, 2, 15),
                        },
                    ]
                },
                "c1/test2.py": {
                    "verifications": [
                        {
                            "status": "skipped",
                            "elapsed": 1,
                            "last_execution_time": datetime.datetime(2020, 2, 15),
                        },
                    ]
                },
            },
        },
        ["c1/a.py", "c1/b1.py", "c1/b2.py", "c1/test.py", "c1/test2.py"],
        {
            "c1/test.py": {
                "path": "c1/test.py",
                "file_input": {
                    "dependencies": ["c1/b1.py"],
                    "verification": [
                        {
                            "type": "const",
                            "status": "success",
                        }
                    ],
                },
                "is_verification": True,
                "verification_status": StatusIcon.TEST_ACCEPTED,
                "verification_results": [
                    {
                        "status": "success",
                        "elapsed": 1,
                        "last_execution_time": datetime.datetime(2020, 2, 15),
                    },
                ],
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": {"c1/b1.py"},
                "required_by": set(),
                "verified_with": set(),
            },
            "c1/test2.py": {
                "path": "c1/test2.py",
                "file_input": {
                    "dependencies": ["c1/b1.py"],
                    "verification": [
                        {
                            "type": "const",
                            "status": "skipped",
                        }
                    ],
                },
                "is_verification": True,
                "verification_status": StatusIcon.TEST_WAITING_JUDGE,
                "verification_results": [
                    {
                        "status": "skipped",
                        "elapsed": 1,
                        "last_execution_time": datetime.datetime(2020, 2, 15),
                    },
                ],
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": {"c1/b1.py"},
                "required_by": set(),
                "verified_with": set(),
            },
            "c1/b1.py": {
                "path": "c1/b1.py",
                "file_input": {"dependencies": ["c1/a.py", "c1/b2.py"]},
                "is_verification": False,
                "verification_status": StatusIcon.LIBRARY_PARTIAL_AC,
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": {"c1/b2.py", "c1/a.py"},
                "required_by": {"c1/b2.py"},
                "verified_with": {"c1/test.py", "c1/test2.py"},
            },
            "c1/b2.py": {
                "path": "c1/b2.py",
                "file_input": {"dependencies": ["c1/b1.py"]},
                "is_verification": False,
                "verification_status": StatusIcon.LIBRARY_PARTIAL_AC,
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": {"c1/b1.py"},
                "required_by": {"c1/b1.py"},
                "verified_with": set(),
            },
            "c1/a.py": {
                "path": "c1/a.py",
                "file_input": {},
                "is_verification": False,
                "verification_status": StatusIcon.LIBRARY_PARTIAL_AC,
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": set(),
                "required_by": {"c1/b1.py"},
                "verified_with": set(),
            },
        },
    ),
    (
        "AC & WA",
        {
            "files": {
                "c1/a.py": {},
                "c1/b1.py": {"dependencies": ["c1/a.py", "c1/b2.py"]},
                "c1/b2.py": {"dependencies": ["c1/b1.py"]},
                "c1/test.py": {
                    "dependencies": ["c1/b1.py"],
                    "verification": [
                        {
                            "type": "const",
                            "status": "success",
                        }
                    ],
                },
                "c1/test2.py": {
                    "dependencies": ["c1/b1.py"],
                    "verification": [
                        {
                            "type": "const",
                            "status": "failure",
                        }
                    ],
                },
            }
        },
        {
            "total_seconds": 10,
            "files": {
                "c1/test.py": {
                    "verifications": [
                        {
                            "status": "success",
                            "elapsed": 1,
                            "last_execution_time": datetime.datetime(2020, 2, 15),
                        },
                    ]
                },
                "c1/test2.py": {
                    "verifications": [
                        {
                            "status": "failure",
                            "elapsed": 1,
                            "last_execution_time": datetime.datetime(2020, 2, 15),
                        },
                    ]
                },
            },
        },
        ["c1/a.py", "c1/b1.py", "c1/b2.py", "c1/test.py", "c1/test2.py"],
        {
            "c1/test.py": {
                "path": "c1/test.py",
                "file_input": {
                    "dependencies": ["c1/b1.py"],
                    "verification": [
                        {
                            "type": "const",
                            "status": "success",
                        }
                    ],
                },
                "is_verification": True,
                "verification_status": StatusIcon.TEST_ACCEPTED,
                "verification_results": [
                    {
                        "status": "success",
                        "elapsed": 1,
                        "last_execution_time": datetime.datetime(2020, 2, 15),
                    },
                ],
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": {"c1/b1.py"},
                "required_by": set(),
                "verified_with": set(),
            },
            "c1/test2.py": {
                "path": "c1/test2.py",
                "file_input": {
                    "dependencies": ["c1/b1.py"],
                    "verification": [
                        {
                            "type": "const",
                            "status": "failure",
                        }
                    ],
                },
                "is_verification": True,
                "verification_status": StatusIcon.TEST_WRONG_ANSWER,
                "verification_results": [
                    {
                        "status": "failure",
                        "elapsed": 1,
                        "last_execution_time": datetime.datetime(2020, 2, 15),
                    },
                ],
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": {"c1/b1.py"},
                "required_by": set(),
                "verified_with": set(),
            },
            "c1/b1.py": {
                "path": "c1/b1.py",
                "file_input": {"dependencies": ["c1/a.py", "c1/b2.py"]},
                "is_verification": False,
                "verification_status": StatusIcon.LIBRARY_SOME_WA,
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": {"c1/b2.py", "c1/a.py"},
                "required_by": {"c1/b2.py"},
                "verified_with": {"c1/test.py", "c1/test2.py"},
            },
            "c1/b2.py": {
                "path": "c1/b2.py",
                "file_input": {"dependencies": ["c1/b1.py"]},
                "is_verification": False,
                "verification_status": StatusIcon.LIBRARY_SOME_WA,
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": {"c1/b1.py"},
                "required_by": {"c1/b1.py"},
                "verified_with": set(),
            },
            "c1/a.py": {
                "path": "c1/a.py",
                "file_input": {},
                "is_verification": False,
                "verification_status": StatusIcon.LIBRARY_SOME_WA,
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": set(),
                "required_by": {"c1/b1.py"},
                "verified_with": set(),
            },
        },
    ),
    (
        "no tests",
        {
            "files": {
                "c1/a.py": {},
                "c1/b1.py": {"dependencies": ["c1/a.py", "c1/b2.py"]},
                "c1/b2.py": {"dependencies": ["c1/b1.py"]},
            }
        },
        {
            "total_seconds": 10,
            "files": {
                "c1/test.py": {
                    "verifications": [
                        {
                            "status": "failure",
                            "elapsed": 1,
                            "last_execution_time": datetime.datetime(2020, 2, 15),
                        },
                    ]
                },
            },
        },
        ["c1/a.py", "c1/b1.py", "c1/b2.py", "c1/test.py"],
        {
            "c1/b1.py": {
                "path": "c1/b1.py",
                "file_input": {"dependencies": ["c1/a.py", "c1/b2.py"]},
                "is_verification": False,
                "verification_status": StatusIcon.LIBRARY_NO_TESTS,
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": {"c1/b2.py", "c1/a.py"},
                "required_by": {"c1/b2.py"},
                "verified_with": set(),
            },
            "c1/b2.py": {
                "path": "c1/b2.py",
                "file_input": {"dependencies": ["c1/b1.py"]},
                "is_verification": False,
                "verification_status": StatusIcon.LIBRARY_NO_TESTS,
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": {"c1/b1.py"},
                "required_by": {"c1/b1.py"},
                "verified_with": set(),
            },
            "c1/a.py": {
                "path": "c1/a.py",
                "file_input": {},
                "is_verification": False,
                "verification_status": StatusIcon.LIBRARY_NO_TESTS,
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": set(),
                "required_by": {"c1/b1.py"},
                "verified_with": set(),
            },
        },
    ),
    (
        "Test depends on lib",
        {
            "files": {
                "c1/hello.java": {
                    "dependencies": ["c1/hello.java", "c1/hello_test.java"]
                },
                "c1/hello_test.java": {
                    "dependencies": ["c1/hello.java", "c1/hello_test.java"],
                    "verification": [
                        {
                            "type": "const",
                            "status": "success",
                        }
                    ],
                },
            }
        },
        {
            "total_seconds": 10,
            "files": {
                "c1/hello_test.java": {
                    "verifications": [
                        {
                            "status": "success",
                            "elapsed": 1,
                            "last_execution_time": datetime.datetime(2020, 2, 15),
                        },
                    ]
                },
            },
        },
        ["c1/hello.java", "c1/hello_test.java"],
        {
            "c1/hello.java": {
                "path": "c1/hello.java",
                "file_input": {"dependencies": ["c1/hello.java", "c1/hello_test.java"]},
                "is_verification": False,
                "verification_status": StatusIcon.LIBRARY_ALL_AC,
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": {"c1/hello_test.java"},
                "required_by": set(),
                "verified_with": {"c1/hello_test.java"},
            },
            "c1/hello_test.java": {
                "path": "c1/hello_test.java",
                "file_input": {
                    "dependencies": ["c1/hello.java", "c1/hello_test.java"],
                    "verification": [
                        {
                            "type": "const",
                            "status": "success",
                        }
                    ],
                },
                "is_verification": True,
                "verification_status": StatusIcon.TEST_ACCEPTED,
                "verification_results": [
                    {
                        "status": "success",
                        "elapsed": 1,
                        "last_execution_time": datetime.datetime(2020, 2, 15),
                    }
                ],
                "timestamp": datetime.datetime(2010, 2, 15, 0, 0),
                "depends_on": {"c1/hello.java"},
                "required_by": {"c1/hello.java"},
                "verified_with": set(),
            },
        },
    ),
]


@pytest.mark.parametrize(
    "name, dep_input_obj, dep_result_obj, included_files_str, expected_obj",
    test_resolve_dependency_params,
    ids=[tup[0] for tup in test_resolve_dependency_params],
)
def test_resolve_dependency(
    name: str,
    dep_input_obj: Any,
    dep_result_obj: Any,
    included_files_str: list[str],
    expected_obj: Any,
):
    with mock.patch(
        "competitive_verifier.git.get_commit_time",
        return_value=datetime.datetime(2010, 2, 15),
    ):
        dep_input = VerificationInput.model_validate(dep_input_obj)
        dep_result = VerifyCommandResult.model_validate(dep_result_obj)

        resolved = SourceCodeStat.resolve_dependency(
            input=dep_input,
            result=dep_result,
            included_files=set(Path(s) for s in included_files_str),
        )
        expected = Parser.model_validate(expected_obj).root
        assert resolved == expected


test_StatusIcon_is_success_params: list[tuple[StatusIcon, bool]] = [
    (StatusIcon.LIBRARY_ALL_AC, True),
    (StatusIcon.LIBRARY_PARTIAL_AC, True),
    (StatusIcon.LIBRARY_SOME_WA, False),
    (StatusIcon.LIBRARY_ALL_WA, False),
    (StatusIcon.LIBRARY_NO_TESTS, True),
    (StatusIcon.TEST_ACCEPTED, True),
    (StatusIcon.TEST_WRONG_ANSWER, False),
    (StatusIcon.TEST_WAITING_JUDGE, True),
]


@pytest.mark.parametrize(
    "status, is_success",
    test_StatusIcon_is_success_params,
)
def test_StatusIcon_is_success(
    status: StatusIcon,
    is_success: bool,
):
    assert status.is_success == is_success
    assert status.is_failed != is_success



import datetime
import pathlib
import textwrap
from io import BytesIO
from typing import Any, Optional

import pytest
import yaml

from competitive_verifier.documents.front_matter import (
    DocumentOutputMode,
    FrontMatter,
    Markdown,
    merge_front_matter,
    split_front_matter,
)
from competitive_verifier.documents.render_data import (
    CategorizedIndex,
    Dependency,
    EmbeddedCode,
    EnvTestcaseResult,
    IndexFiles,
    IndexRenderData,
    PageRenderData,
    RenderLink,
    StatusIcon,
)
from competitive_verifier.models import JudgeStatus

test_markdown_params: list[tuple[bytes, Optional[dict[str, Any]], bytes]] = [
    (
        b"""---
---""",
        {},
        b"",
    ),
    (
        b"""---
title: A + B
documentation_of: //examples/awk/aplusb2.awk
---""",
        {
            "documentation_of": "//examples/awk/aplusb2.awk",
            "title": "A + B",
        },
        b"",
    ),
    (
        b"""---
---
""",
        {},
        b"",
    ),
    (
        b"""---
title: A + B
documentation_of: //examples/awk/aplusb2.awk
---
""",
        {
            "documentation_of": "//examples/awk/aplusb2.awk",
            "title": "A + B",
        },
        b"",
    ),
    (
        b"""---
---
# aplusb2

String $A + B$""",
        {},
        b"""# aplusb2

String $A + B$""",
    ),
    (
        b"""---
title: A + B
documentation_of: //examples/awk/aplusb2.awk
---
# aplusb2

String $A + B$""",
        {
            "documentation_of": "//examples/awk/aplusb2.awk",
            "title": "A + B",
        },
        b"""# aplusb2

String $A + B$""",
    ),
    (
        b"""---
title: A + B
documentation_of: //examples/awk/aplusb2.awk""",
        None,
        b"""---
title: A + B
documentation_of: //examples/awk/aplusb2.awk""",
    ),
    (
        b"""
title: A + B
documentation_of: //examples/awk/aplusb2.awk""",
        None,
        b"""
title: A + B
documentation_of: //examples/awk/aplusb2.awk""",
    ),
    (
        b"""---
title: A + B
display: no-index
additional-extra-property: "CDE"
---
""",
        {
            "title": "A + B",
            "display": "no-index",
            "additional-extra-property": "CDE",
        },
        b"",
    ),
]


@pytest.mark.parametrize(
    "content, front_matter, markdown_content",
    test_markdown_params,
    ids=range(len(test_markdown_params)),
)
def test_markdown(
    content: bytes,
    front_matter: Optional[dict[str, Any]],
    markdown_content: bytes,
):
    with BytesIO(content) as fp:
        md = Markdown.load(fp)
    expected = Markdown(
        front_matter=front_matter,  # pyright:ignore
        content=markdown_content,
    )
    assert md == expected

    actual_front_matter, actual_content = split_front_matter(content)
    assert actual_content == markdown_content

    if actual_front_matter:
        assert yaml.safe_load(actual_front_matter.model_dump_yml()) == front_matter
    else:
        assert actual_front_matter is None and front_matter is None

    with BytesIO() as fp:
        merge_front_matter(fp, front_matter=actual_front_matter, content=actual_content)
        fp.seek(0)
        merged = fp.read()
        assert merged.endswith(markdown_content)
        if front_matter is not None:
            front = merged[: -len(markdown_content)] if markdown_content else merged
            assert front.startswith(b"---\n")
            assert front.endswith(b"---\n")
            seplen = len(b"---\n")
            assert yaml.safe_load(front[seplen:-seplen]) == front_matter

    with BytesIO() as fp:
        md.dump_merged(fp)
        fp.seek(0)
        assert fp.read() == merged


test_front_matter_dump_yml_params: list[tuple[FrontMatter, bytes]] = [
    (
        FrontMatter(),
        textwrap.dedent(
            """{}
            """
        )
        .lstrip()
        .encode("utf-8"),
    ),
    (
        FrontMatter(
            documentation_of="../foo.txt",
            layout="document",
            redirect_from=["bar.txt"],
            title="Foooo",
            data=IndexRenderData(
                top=[
                    IndexFiles(
                        type="Lib",
                        categories=[
                            CategorizedIndex(
                                name="/a/b/1",
                                pages=[
                                    RenderLink(
                                        path=pathlib.Path("/a/b/1/d.txt"),
                                        filename="d.txt",
                                        title="D(text)",
                                        icon=StatusIcon.LIBRARY_ALL_AC,
                                    ),
                                    RenderLink(
                                        path=pathlib.Path("/a/b/1/e.txt"),
                                        filename="e.txt",
                                        icon=StatusIcon.LIBRARY_NO_TESTS,
                                    ),
                                    RenderLink(
                                        path=pathlib.Path("/a/b/1/f.txt"),
                                        filename="f.txt",
                                        icon=StatusIcon.LIBRARY_PARTIAL_AC,
                                    ),
                                ],
                            ),
                            CategorizedIndex(
                                name="/a/b/c",
                                pages=[
                                    RenderLink(
                                        path=pathlib.Path("/a/b/c/d.txt"),
                                        filename="d.txt",
                                        title="D(text)",
                                        icon=StatusIcon.LIBRARY_SOME_WA,
                                    ),
                                    RenderLink(
                                        path=pathlib.Path("/a/b/c/e.txt"),
                                        filename="e.txt",
                                        icon=StatusIcon.LIBRARY_ALL_WA,
                                    ),
                                ],
                            ),
                        ],
                    ),
                    IndexFiles(
                        type="Verification",
                        categories=[
                            CategorizedIndex(
                                name="/z/b/1",
                                pages=[
                                    RenderLink(
                                        path=pathlib.Path("/z/b/1/d.txt"),
                                        filename="d.txt",
                                        title="D(text)",
                                        icon=StatusIcon.TEST_ACCEPTED,
                                    ),
                                    RenderLink(
                                        path=pathlib.Path("/z/b/1/e.txt"),
                                        filename="e.txt",
                                        icon=StatusIcon.TEST_WAITING_JUDGE,
                                    ),
                                    RenderLink(
                                        path=pathlib.Path("/z/b/1/f.txt"),
                                        filename="f.txt",
                                        icon=StatusIcon.TEST_WAITING_JUDGE,
                                    ),
                                ],
                            ),
                            CategorizedIndex(
                                name="/z/b/c",
                                pages=[
                                    RenderLink(
                                        path=pathlib.Path("/z/b/c/d.txt"),
                                        filename="d.txt",
                                        title="D(text)",
                                        icon=StatusIcon.TEST_WRONG_ANSWER,
                                    ),
                                    RenderLink(
                                        path=pathlib.Path("/z/b/c/e.txt"),
                                        filename="e.txt",
                                        icon=StatusIcon.TEST_WAITING_JUDGE,
                                    ),
                                ],
                            ),
                        ],
                    ),
                ]
            ),
        ),
        textwrap.dedent(
            """
            data:
              top:
              - categories:
                - name: /a/b/1
                  pages:
                  - filename: d.txt
                    icon: LIBRARY_ALL_AC
                    path: /a/b/1/d.txt
                    title: D(text)
                  - filename: e.txt
                    icon: LIBRARY_NO_TESTS
                    path: /a/b/1/e.txt
                  - filename: f.txt
                    icon: LIBRARY_PARTIAL_AC
                    path: /a/b/1/f.txt
                - name: /a/b/c
                  pages:
                  - filename: d.txt
                    icon: LIBRARY_SOME_WA
                    path: /a/b/c/d.txt
                    title: D(text)
                  - filename: e.txt
                    icon: LIBRARY_ALL_WA
                    path: /a/b/c/e.txt
                type: Lib
              - categories:
                - name: /z/b/1
                  pages:
                  - filename: d.txt
                    icon: TEST_ACCEPTED
                    path: /z/b/1/d.txt
                    title: D(text)
                  - filename: e.txt
                    icon: TEST_WAITING_JUDGE
                    path: /z/b/1/e.txt
                  - filename: f.txt
                    icon: TEST_WAITING_JUDGE
                    path: /z/b/1/f.txt
                - name: /z/b/c
                  pages:
                  - filename: d.txt
                    icon: TEST_WRONG_ANSWER
                    path: /z/b/c/d.txt
                    title: D(text)
                  - filename: e.txt
                    icon: TEST_WAITING_JUDGE
                    path: /z/b/c/e.txt
                type: Verification
            documentation_of: ../foo.txt
            layout: document
            redirect_from:
            - bar.txt
            title: Foooo
            """
        )
        .lstrip()
        .encode("utf-8"),
    ),
    (
        FrontMatter(
            documentation_of="../foo.txt",
            layout="document",
            redirect_from=["bar.txt"],
            title="Foooo",
            data=PageRenderData(
                title=None,
                path=pathlib.Path("root/foo.txt"),
                path_extension="txt",
                document_path=pathlib.Path("root/docs/foo.md"),
                embedded=[
                    EmbeddedCode(
                        name="default",
                        code="👍",
                    ),
                ],
                timestamp=datetime.datetime(
                    2006,
                    1,
                    2,
                    13,
                    46,
                    57,
                    433,
                    tzinfo=datetime.timezone(datetime.timedelta(hours=9)),
                ),
                attributes={
                    "PROBLEM": "http://example.com",
                },
                testcases=[
                    EnvTestcaseResult(
                        name="test1",
                        status=JudgeStatus.AC,
                        elapsed=1.2,
                        environment="tx",
                        memory=120.2,
                    ),
                ],
                is_failed=False,
                is_verification_file=True,
                verification_status=StatusIcon.TEST_ACCEPTED,
                dependencies=[
                    Dependency(
                        type="depends_on",
                        files=[
                            RenderLink(
                                path=pathlib.Path("dep/a.txt"),
                                filename="a.txt",
                                icon=StatusIcon.LIBRARY_ALL_AC,
                                title="A(txt)",
                            ),
                            RenderLink(
                                path=pathlib.Path("dep/b.txt"),
                                filename="b.txt",
                                icon=StatusIcon.LIBRARY_ALL_AC,
                            ),
                        ],
                    ),
                    Dependency(
                        type="required_by",
                        files=[
                            RenderLink(
                                path=pathlib.Path("req/a.txt"),
                                filename="a.txt",
                                icon=StatusIcon.TEST_ACCEPTED,
                                title="A(txt)",
                            ),
                            RenderLink(
                                path=pathlib.Path("req/b.txt"),
                                filename="b.txt",
                                icon=StatusIcon.TEST_WAITING_JUDGE,
                            ),
                        ],
                    ),
                    Dependency(
                        type="verified_with",
                        files=[
                            RenderLink(
                                path=pathlib.Path("ver/a.txt"),
                                filename="a.txt",
                                icon=StatusIcon.TEST_ACCEPTED,
                                title="A(txt)",
                            ),
                            RenderLink(
                                path=pathlib.Path("ver/b.txt"),
                                filename="b.txt",
                                icon=StatusIcon.TEST_WAITING_JUDGE,
                            ),
                        ],
                    ),
                ],
                depends_on=[
                    pathlib.Path("dep/a.txt"),
                    pathlib.Path("dep/b.txt"),
                ],
                required_by=[
                    pathlib.Path("req/a.txt"),
                    pathlib.Path("req/b.txt"),
                ],
                verified_with=[
                    pathlib.Path("ver/a.txt"),
                    pathlib.Path("ver/b.txt"),
                ],
            ),
        ),
        textwrap.dedent(
            """
            data:
              attributes:
                PROBLEM: http://example.com
              dependencies:
              - files:
                - filename: a.txt
                  icon: LIBRARY_ALL_AC
                  path: dep/a.txt
                  title: A(txt)
                - filename: b.txt
                  icon: LIBRARY_ALL_AC
                  path: dep/b.txt
                type: depends_on
              - files:
                - filename: a.txt
                  icon: TEST_ACCEPTED
                  path: req/a.txt
                  title: A(txt)
                - filename: b.txt
                  icon: TEST_WAITING_JUDGE
                  path: req/b.txt
                type: required_by
              - files:
                - filename: a.txt
                  icon: TEST_ACCEPTED
                  path: ver/a.txt
                  title: A(txt)
                - filename: b.txt
                  icon: TEST_WAITING_JUDGE
                  path: ver/b.txt
                type: verified_with
              dependsOn:
              - dep/a.txt
              - dep/b.txt
              documentPath: root/docs/foo.md
              embedded:
              - code: "\\U0001F44D"
                name: default
              isFailed: false
              isVerificationFile: true
              path: root/foo.txt
              pathExtension: txt
              requiredBy:
              - req/a.txt
              - req/b.txt
              testcases:
              - elapsed: 1.2
                environment: tx
                memory: 120.2
                name: test1
                status: AC
              timestamp: '2006-01-02 13:46:57.000433+09:00'
              verificationStatus: TEST_ACCEPTED
              verifiedWith:
              - ver/a.txt
              - ver/b.txt
            documentation_of: ../foo.txt
            layout: document
            redirect_from:
            - bar.txt
            title: Foooo
            """
        )
        .lstrip()
        .encode("utf-8"),
    ),
    (
        FrontMatter(display=DocumentOutputMode.visible),
        textwrap.dedent(
            """display: visible
            """
        )
        .lstrip()
        .encode("utf-8"),
    ),
    (
        FrontMatter(display=DocumentOutputMode.hidden),
        textwrap.dedent(
            """display: hidden
            """
        )
        .lstrip()
        .encode("utf-8"),
    ),
    (
        FrontMatter(display=DocumentOutputMode.no_index),
        textwrap.dedent(
            """display: no-index
            """
        )
        .lstrip()
        .encode("utf-8"),
    ),
    (
        FrontMatter(display=DocumentOutputMode.never),
        textwrap.dedent(
            """display: never
            """
        )
        .lstrip()
        .encode("utf-8"),
    ),
]


@pytest.mark.parametrize(
    "front_matter, expected",
    test_front_matter_dump_yml_params,
    ids=range(len(test_front_matter_dump_yml_params)),
)
def test_front_matter_dump_yml(front_matter: FrontMatter, expected: bytes):
    assert front_matter.model_dump_yml() == expected



# flake8: noqa E501
import pathlib
import sys
from dataclasses import dataclass
from typing import Any, Optional, Sequence, Union

import pytest
from pydantic import RootModel
from pytest_mock import MockerFixture
from pytest_mock.plugin import MockType

import competitive_verifier.models
import competitive_verifier.oj.tools.test_command
from competitive_verifier.models import (
    CommandVerification,
    ConstVerification,
    ProblemVerification,
    ResultStatus,
    ShellCommand,
    Verification,
)
from competitive_verifier.oj.tools.test_command import OjTestArguments


@dataclass
class DataVerificationParams:
    default_tle: Optional[float]
    default_mle: Optional[float]


test_command_union_json_params: list[tuple[Verification, str, str]] = [
    (
        ConstVerification(status=ResultStatus.SUCCESS),
        '{"type": "const","status":"success"}',
        '{"type":"const","status":"success"}',
    ),
    (
        ConstVerification(status=ResultStatus.SUCCESS, name="foo"),
        '{"type": "const","name":"foo","status":"success"}',
        '{"name":"foo","type":"const","status":"success"}',
    ),
    (
        ConstVerification(status=ResultStatus.FAILURE),
        '{"type": "const","status":"failure"}',
        '{"type":"const","status":"failure"}',
    ),
    (
        ConstVerification(status=ResultStatus.SKIPPED),
        '{"type": "const","status":"skipped"}',
        '{"type":"const","status":"skipped"}',
    ),
    (
        ConstVerification(status=ResultStatus.SUCCESS),
        '{"type": "const","status":"success"}',
        '{"type":"const","status":"success"}',
    ),
    (
        CommandVerification(command="ls ~"),
        '{"type":"command","command":"ls ~"}',
        '{"type":"command","command":"ls ~"}',
    ),
    (
        CommandVerification(compile="cat LICENSE", command="ls ~"),
        '{"type": "command", "compile": "cat LICENSE", "command": "ls ~"}',
        '{"type":"command","command":"ls ~","compile":"cat LICENSE"}',
    ),
    (
        ProblemVerification(command="ls ~", problem="https://example.com"),
        '{"type":"problem","problem":"https://example.com","command":"ls ~"}',
        '{"type":"problem","command":"ls ~","problem":"https://example.com"}',
    ),
    (
        ProblemVerification(name="bar", command="ls ~", problem="https://example.com"),
        '{"type":"problem","name":"bar","problem":"https://example.com","command":"ls ~"}',
        '{"name":"bar","type":"problem","command":"ls ~","problem":"https://example.com"}',
    ),
    (
        ProblemVerification(
            compile="cat LICENSE", command="ls ~", problem="https://example.com"
        ),
        '{"type": "problem",  "problem":"https://example.com", "compile": "cat LICENSE", "command": "ls ~"}',
        '{"type":"problem","command":"ls ~","compile":"cat LICENSE","problem":"https://example.com"}',
    ),
    (
        ProblemVerification(
            compile="cat LICENSE",
            command="ls ~",
            problem="https://example.com",
            error=1e-6,
            tle=2,
        ),
        '{"type": "problem",  "problem":"https://example.com", "compile": "cat LICENSE", "error": 0.000001, "tle": 2, "command": "ls ~"}',
        '{"type":"problem","command":"ls ~","compile":"cat LICENSE","problem":"https://example.com","error":1e-6,"tle":2.0}',
    ),
]


@pytest.mark.parametrize("obj, raw_json, output_json", test_command_union_json_params)
def test_command_union_json(
    obj: Verification,
    raw_json: str,
    output_json: str,
):
    VerificationUnion = RootModel[Verification]

    assert obj == type(obj).model_validate_json(raw_json)
    assert obj.model_dump_json(exclude_none=True) == output_json

    assert obj == VerificationUnion.model_validate_json(raw_json).root


def test_const_verification(mock_exec_command: MockType):
    obj = ConstVerification(status=ResultStatus.SUCCESS)

    assert obj.run() == ResultStatus.SUCCESS
    assert obj.run_compile_command()
    mock_exec_command.assert_not_called()


test_run_params: list[
    tuple[Verification, tuple[Union[str, list[str]]], dict[str, Any]]
] = [
    (
        CommandVerification(command="ls ~"),
        ("ls ~",),
        {
            "shell": True,
            "text": True,
            "check": False,
            "cwd": None,
            "env": None,
            "capture_output": False,
            "encoding": sys.stdout.encoding,
        },
    ),
    (
        CommandVerification(compile="cat LICENSE", command="ls ~"),
        ("ls ~",),
        {
            "shell": True,
            "text": True,
            "check": False,
            "cwd": None,
            "env": None,
            "capture_output": False,
            "encoding": sys.stdout.encoding,
        },
    ),
    (
        CommandVerification(command=["ls", "~"]),
        (["ls", "~"],),
        {
            "shell": False,
            "text": True,
            "check": False,
            "cwd": None,
            "env": None,
            "capture_output": False,
            "encoding": sys.stdout.encoding,
        },
    ),
    (
        CommandVerification(command=["ls", "~"]),
        (["ls", "~"],),
        {
            "shell": False,
            "text": True,
            "check": False,
            "cwd": None,
            "env": None,
            "capture_output": False,
            "encoding": sys.stdout.encoding,
        },
    ),
    (
        CommandVerification(
            command=ShellCommand(command="ls ~", cwd=pathlib.Path("/foo")),
        ),
        ("ls ~",),
        {
            "shell": True,
            "text": True,
            "check": False,
            "cwd": pathlib.Path("/foo"),
            "env": None,
            "capture_output": False,
            "encoding": sys.stdout.encoding,
        },
    ),
    (
        CommandVerification(
            command=ShellCommand(command=["ls", "~"], cwd=pathlib.Path("~")),
        ),
        (["ls", "~"],),
        {
            "shell": False,
            "text": True,
            "check": False,
            "cwd": pathlib.Path("~"),
            "env": None,
            "capture_output": False,
            "encoding": sys.stdout.encoding,
        },
    ),
]


@pytest.mark.parametrize(
    "obj, args, kwargs",
    test_run_params,
    ids=range(len(test_run_params)),
)
def test_run(
    obj: Verification,
    args: Sequence[Any],
    kwargs: dict[str, Any],
    mock_exec_command: MockType,
    mocker: MockerFixture,
):
    mocker.patch(
        "onlinejudge._implementation.utils.user_cache_dir", pathlib.Path("/bar/baz")
    )
    obj.run(
        DataVerificationParams(
            default_tle=22,
            default_mle=128,
        ),
    )
    mock_exec_command.assert_called_once_with(*args, **kwargs)


test_run_with_env_params: list[
    tuple[Verification, tuple[Union[str, list[str]]], dict[str, Any], dict[str, str]]
] = [
    (
        CommandVerification(
            command=ShellCommand(command="ls ~", env={"TOKEN": "DUMMY"}),
        ),
        ("ls ~",),
        {
            "shell": True,
            "text": True,
            "check": False,
            "cwd": None,
            "capture_output": False,
            "encoding": sys.stdout.encoding,
        },
        {"TOKEN": "DUMMY"},
    ),
    (
        CommandVerification(
            command=ShellCommand(command=["ls", "~"], env={"TOKEN": "DUMMY"}),
        ),
        (["ls", "~"],),
        {
            "shell": False,
            "text": True,
            "check": False,
            "cwd": None,
            "capture_output": False,
            "encoding": sys.stdout.encoding,
        },
        {"TOKEN": "DUMMY"},
    ),
]


@pytest.mark.parametrize(
    "obj, args, kwargs, env",
    test_run_with_env_params,
    ids=range(len(test_run_with_env_params)),
)
def test_run_with_env(
    obj: Verification,
    args: Sequence[Any],
    kwargs: dict[str, Any],
    env: dict[str, str],
    mock_exec_command: MockType,
    mocker: MockerFixture,
):
    mocker.patch(
        "onlinejudge._implementation.utils.user_cache_dir", pathlib.Path("/bar/baz")
    )
    obj.run(
        DataVerificationParams(
            default_tle=22,
            default_mle=128,
        ),
    )
    mock_exec_command.assert_called_once()
    assert mock_exec_command.call_args[0] == args

    call_kwargs: dict[str, Any] = mock_exec_command.call_args[1]
    assert dict(call_kwargs["env"].items() & env.items()) == env
    assert len(call_kwargs["env"]) > len(env)
    del call_kwargs["env"]
    assert call_kwargs == kwargs


test_run_problem_command_params: list[tuple[ProblemVerification, OjTestArguments]] = [
    (
        ProblemVerification(command="ls ~", problem="https://example.com"),
        OjTestArguments(
            cookie=pathlib.Path("/any/cache/cookie.txt"),
            directory=pathlib.Path("/any/test"),
            judge=None,
            command="ls ~",
            tle=22.0,
            error=None,
            mle=128,
            env=None,
        ),
    ),
    (
        ProblemVerification(
            compile="cat LICENSE", command="ls ~", problem="https://example.com"
        ),
        OjTestArguments(
            cookie=pathlib.Path("/any/cache/cookie.txt"),
            directory=pathlib.Path("/any/test"),
            judge=None,
            command="ls ~",
            tle=22.0,
            error=None,
            mle=128,
            env=None,
        ),
    ),
    (
        ProblemVerification(
            compile="cat LICENSE",
            command="ls ~",
            problem="https://example.com",
            error=1e-6,
            tle=2,
            mle=1.2,
        ),
        OjTestArguments(
            cookie=pathlib.Path("/any/cache/cookie.txt"),
            directory=pathlib.Path("/any/test"),
            judge=None,
            command="ls ~",
            tle=2.0,
            error=1e-06,
            mle=1.2,
            env=None,
        ),
    ),
    (
        ProblemVerification(
            compile=ShellCommand(command="cat LICENSE", env={"ARG": "VAR"}),
            command="ls ~",
            problem="https://judge.yosupo.jp/problem/aplusb",
            error=1e-6,
            tle=2,
        ),
        OjTestArguments(
            cookie=pathlib.Path("/any/cache/cookie.txt"),
            directory=pathlib.Path("/any/test"),
            judge=None,
            command="ls ~",
            tle=2.0,
            error=1e-06,
            mle=128,
            env=None,
        ),
    ),
    (
        ProblemVerification(
            compile="cat LICENSE",
            command=ShellCommand(command="ls ~", env={"ARG": "VAR"}),
            problem="https://judge.yosupo.jp/problem/aplusb",
            error=1e-6,
            tle=2,
        ),
        OjTestArguments(
            cookie=pathlib.Path("/any/cache/cookie.txt"),
            directory=pathlib.Path("/any/test"),
            judge=None,
            command="ls ~",
            tle=2.0,
            error=1e-06,
            mle=128,
            env={"ARG": "VAR"},
        ),
    ),
]


@pytest.mark.parametrize(
    "obj, args",
    test_run_problem_command_params,
    ids=range(len(test_run_problem_command_params)),
)
def test_run_problem_command(
    obj: ProblemVerification,
    args: OjTestArguments,
    mocker: MockerFixture,
):
    patch = mocker.patch.object(competitive_verifier.oj.tools.test_command, "run")

    mocker.patch(
        "competitive_verifier.oj.tools.test_command.get_cache_directory",
        return_value=pathlib.Path("/any/cache"),
    )
    mocker.patch(
        "competitive_verifier.oj.tools.test_command.get_directory",
        return_value=pathlib.Path("/any/"),
    )

    obj.run(
        DataVerificationParams(default_tle=22, default_mle=128),
    )
    patch.assert_called_once_with(args)


test_run_compile_params: list[
    tuple[Verification, Optional[Union[str, list[str]]], Optional[dict[str, Any]]]
] = [
    (
        CommandVerification(command="ls ~"),
        None,
        None,
    ),
    (
        CommandVerification(compile="cat LICENSE", command="ls ~"),
        "cat LICENSE",
        {
            "shell": True,
            "text": True,
            "check": False,
            "env": None,
            "cwd": None,
            "capture_output": False,
            "encoding": sys.stdout.encoding,
        },
    ),
    (
        ProblemVerification(command="ls ~", problem="https://example.com"),
        None,
        None,
    ),
    (
        ProblemVerification(
            compile="cat LICENSE", command="ls ~", problem="https://example.com"
        ),
        "cat LICENSE",
        {
            "shell": True,
            "text": True,
            "check": False,
            "env": None,
            "cwd": None,
            "capture_output": False,
            "encoding": sys.stdout.encoding,
        },
    ),
    (
        ProblemVerification(
            compile="cat LICENSE",
            command="ls ~",
            problem="https://example.com",
            error=1e-6,
            tle=2,
        ),
        "cat LICENSE",
        {
            "shell": True,
            "text": True,
            "check": False,
            "env": None,
            "cwd": None,
            "capture_output": False,
            "encoding": sys.stdout.encoding,
        },
    ),
    (
        ProblemVerification(
            compile=["cat", "LICENSE"],
            command="ls ~",
            problem="https://example.com",
            error=1e-6,
            tle=2,
        ),
        ["cat", "LICENSE"],
        {
            "shell": False,
            "text": True,
            "check": False,
            "env": None,
            "cwd": None,
            "capture_output": False,
            "encoding": sys.stdout.encoding,
        },
    ),
    (
        ProblemVerification(
            compile=ShellCommand(
                command=["cat", "LICENSE"],
                cwd=pathlib.Path("~/foo"),
            ),
            command="ls ~",
            problem="https://example.com",
            error=1e-6,
            tle=2,
        ),
        ["cat", "LICENSE"],
        {
            "shell": False,
            "text": True,
            "check": False,
            "env": None,
            "cwd": pathlib.Path("~/foo"),
            "capture_output": False,
            "encoding": sys.stdout.encoding,
        },
    ),
]


@pytest.mark.parametrize(
    "obj, args, kwargs",
    test_run_compile_params,
    ids=range(len(test_run_compile_params)),
)
def test_run_compile(
    obj: Verification,
    args: Optional[Sequence[Any]],
    kwargs: Optional[dict[str, Any]],
    mock_exec_command: MockType,
):
    obj.run_compile_command(
        DataVerificationParams(
            default_tle=22,
            default_mle=128,
        ),
    )
    if args is None:
        mock_exec_command.assert_not_called()
    else:
        assert kwargs
        mock_exec_command.assert_called_once_with(args, **kwargs)


test_params_run_params: list[tuple[Verification, Optional[str]]] = [
    (ConstVerification(status=ResultStatus.SUCCESS), None),
    (CommandVerification(command="true"), None),
    (
        ProblemVerification(command="true", problem="http://example.com"),
        "ProblemVerification.run requires VerificationParams",
    ),
]


@pytest.mark.parametrize(
    "obj, error_message",
    test_params_run_params,
    ids=range(len(test_params_run_params)),
)
def test_params_run(
    obj: Verification,
    error_message: Optional[str],
    mock_exec_command: MockType,
):
    if error_message:
        with pytest.raises(ValueError) as e:
            obj.run()
        assert e.value.args == (error_message,)
    else:
        obj.run()



import importlib.resources
from typing import Iterable

_RESOURCE_PACKAGE = "competitive_verifier_resources"
_RESOURCE_STATIC_FILE_PATHS: list[str] = [
    "_layouts/page.html",
    "_layouts/document.html",
    "_layouts/multidoc.html",
    "_layouts/toppage.html",
    "_includes/head-custom.html",
    "_includes/head-custom2.html",
    "_includes/mathjax/mathjax.html",
    "_includes/mathjax/mathjax2.html",
    "_includes/mathjax/mathjax3.html",
    "_includes/code.html",
    "_includes/code_and_testcases.html",
    "_includes/highlight_additional.html",
    "_includes/highlight/highlight_header.html",
    "_includes/dependencies.html",
    "_includes/document_header.html",
    "_includes/document_body.html",
    "_includes/document_footer.html",
    "_includes/multidoc_body.html",
    "_includes/multidoc_header.html",
    "_includes/toppage_header.html",
    "_includes/toppage_body.html",
    "assets/css/default.scss",
    "assets/css/code.scss",
    "assets/js/code.js",
    "Gemfile",
]


def default_resource_files() -> Iterable[tuple[str, bytes]]:
    for path in _RESOURCE_STATIC_FILE_PATHS:
        yield path, (
            importlib.resources.files(_RESOURCE_PACKAGE) / "jekyll" / path
        ).read_bytes()
